@@ -1,7 +1,12 @@
-import contextlib, datetime, multiprocessing, os, pathlib, subprocess, tempfile, time, unittest
-from job_lock import clean_up_old_job_locks, clear_slurm_running_jobs_cache, JobLock, JobLockAndWait, jobinfo, MultiJobLock, setsqueueoutput, slurm_clean_up_temp_dir, slurm_rsync_input, slurm_rsync_output
+import argparse, contextlib, datetime, logging, multiprocessing, os, pathlib, subprocess, sys, tempfile, time, unittest
+from job_lock import clean_up_old_job_locks, clear_running_jobs_cache, JobLock, JobLockAndWait, jobinfo, MultiJobLock, setsqueueoutput, slurm_clean_up_temp_dir, slurm_rsync_input, slurm_rsync_output
+from job_lock.job_lock import clean_up_old_job_locks_argparse
+
+logger = logging.getLogger("JobLock")
 
 class TestJobLock(unittest.TestCase, contextlib.ExitStack):
+  loglevel = logging.INFO
+
   def __init__(self, *args, **kwargs):
     super().__init__(*args, **kwargs)
     try:
@@ -10,13 +15,16 @@ class TestJobLock(unittest.TestCase, contextlib.ExitStack):
       contextlib.ExitStack.__init__(self)
     self.maxDiff = None
   def setUp(self):
+    super().setUp()
     self.tmpdir = pathlib.Path(self.enter_context(tempfile.TemporaryDirectory()))
     self.bkpenviron = os.environ.copy()
     self.slurm_tmpdir = self.tmpdir/"slurm_tmpdir"
     self.slurm_tmpdir.mkdir()
     os.environ["TMPDIR"] = os.fspath(self.slurm_tmpdir)
-    clear_slurm_running_jobs_cache()
+    clear_running_jobs_cache()
     setsqueueoutput()
+    logger.setLevel(self.loglevel)
+    JobLock.setdefaultcorruptfiletimeout(None)
   def tearDown(self):
     del self.tmpdir
     self.close()
@@ -24,12 +32,12 @@ class TestJobLock(unittest.TestCase, contextlib.ExitStack):
     os.environ.update(self.bkpenviron)
 
   def testJobLock(self):
-    with JobLock(self.tmpdir/"lock1.lock") as lock1:
+    with JobLock(self.tmpdir/"lock1.lck") as lock1:
       self.assertTrue(lock1)
-      self.assertEqual(lock1.iterative_lock_filename, self.tmpdir/"lock1.lock_2")
-      with JobLock(self.tmpdir/"lock2.lock") as lock2:
+      self.assertEqual(lock1.iterative_lock_filename, self.tmpdir/"lock1.lck.lock")
+      with JobLock(self.tmpdir/"lock2.lck") as lock2:
         self.assertTrue(lock2)
-      with JobLock(self.tmpdir/"lock1.lock") as lock3:
+      with JobLock(self.tmpdir/"lock1.lck") as lock3:
         self.assertFalse(lock3)
         self.assertEqual(lock3.debuginfo, {"outputsexist": None, "inputsexist": None, "prevsteplockfilesexist": None, "oldjobinfo": jobinfo(), "removed_failed_job": False, "iterative_lock_debuginfo": {"inputsexist": None, "iterative_lock_debuginfo": None, "oldjobinfo": None, "outputsexist": None, "prevsteplockfilesexist": None, "removed_failed_job": False}})
 
@@ -217,28 +225,16 @@ class TestJobLock(unittest.TestCase, contextlib.ExitStack):
     with open(self.tmpdir/"lock1.lock", "w") as f:
       f.write("SLURM 0 1234567")
     with open(self.tmpdir/"lock2.lock", "w") as f:
-      f.write("1234567")
-    with open(self.tmpdir/"lock3.lock", "w") as f:
       f.write("SLURM 0 12345678")
-    with open(self.tmpdir/"lock4.lock", "w") as f:
-      f.write("12345678")
-    with open(self.tmpdir/"lock5.lock", "w") as f:
+    with open(self.tmpdir/"lock3.lock", "w") as f:
       f.write("SLURM 0 1234568")
-    with open(self.tmpdir/"lock6.lock", "w") as f:
-      f.write("1234568")
 
     with JobLock(self.tmpdir/"lock1.lock") as lock1:
       self.assertFalse(lock1)
     with JobLock(self.tmpdir/"lock2.lock") as lock2:
-      self.assertFalse(lock2)
+      self.assertTrue(lock2)
     with JobLock(self.tmpdir/"lock3.lock") as lock3:
       self.assertTrue(lock3)
-    with JobLock(self.tmpdir/"lock4.lock") as lock4:
-      self.assertTrue(lock4)
-    with JobLock(self.tmpdir/"lock5.lock") as lock5:
-      self.assertTrue(lock5)
-    with JobLock(self.tmpdir/"lock6.lock") as lock6:
-      self.assertTrue(lock6)
 
   def testcondor(self):
     dummycondor_q = """
@@ -369,23 +365,26 @@ class TestJobLock(unittest.TestCase, contextlib.ExitStack):
     time.sleep(1)
     with open(self.tmpdir/"lock1.lock_10", "w"): pass
     with open(self.tmpdir/"lock1.lock", "w"): pass
-    with JobLock(self.tmpdir/"lock1.lock", corruptfiletimeout=datetime.timedelta(seconds=1)) as lock:
+    with open(self.tmpdir/"output.txt", "w"): pass
+    with JobLock(self.tmpdir/"lock1.lock", corruptfiletimeout=datetime.timedelta(seconds=1), outputfiles=[self.tmpdir/"output.txt"]) as lock:
       self.assertFalse(lock)
       self.assertIsInstance(lock.debuginfo["oldjobinfo"], ValueError)
     self.assertFalse((self.tmpdir/"lock1.lock_2").exists())
     self.assertTrue((self.tmpdir/"lock1.lock_5").exists())
     self.assertTrue((self.tmpdir/"lock1.lock_10").exists())
     self.assertTrue((self.tmpdir/"lock1.lock_30").exists())
+    self.assertTrue((self.tmpdir/"output.txt").exists())
     time.sleep(1)
-    with JobLock(self.tmpdir/"lock1.lock") as lock:
+    with JobLock(self.tmpdir/"lock1.lock", outputfiles=[self.tmpdir/"output.txt"]) as lock:
       self.assertFalse(lock)
-    with JobLock(self.tmpdir/"lock1.lock", corruptfiletimeout=datetime.timedelta(seconds=10)) as lock:
+    with JobLock(self.tmpdir/"lock1.lock", corruptfiletimeout=datetime.timedelta(seconds=10), outputfiles=[self.tmpdir/"output.txt"]) as lock:
       self.assertFalse(lock)
-    with JobLock(self.tmpdir/"lock1.lock", corruptfiletimeout=datetime.timedelta(seconds=1)) as lock:
+    with JobLock(self.tmpdir/"lock1.lock", corruptfiletimeout=datetime.timedelta(seconds=1), outputfiles=[self.tmpdir/"output.txt"]) as lock:
       self.assertTrue(lock)
     self.assertFalse((self.tmpdir/"lock1.lock_5").exists())
     self.assertFalse((self.tmpdir/"lock1.lock_10").exists())
     self.assertFalse((self.tmpdir/"lock1.lock_30").exists())
+    self.assertFalse((self.tmpdir/"output.txt").exists())
 
   def testCleanUp(self):
     with open(self.tmpdir/"lock1.lock_2", "w"): pass
@@ -400,13 +399,13 @@ class TestJobLock(unittest.TestCase, contextlib.ExitStack):
     self.assertTrue((self.tmpdir/"lock1.lock_30").exists())
     time.sleep(1)
     clean_up_old_job_locks(self.tmpdir, howold=datetime.timedelta(seconds=1), dryrun=True, silent=True)
-    subprocess.run(["clean_up_old_job_locks", self.tmpdir, "--hours-old", str(1/3600), "--silent", "--dry-run"], check=True)
+    clean_up_old_job_locks_argparse(["clean_up_old_job_locks", os.fspath(self.tmpdir), "--hours-old", str(1/3600), "--silent", "--dry-run"])
     self.assertTrue((self.tmpdir/"lock1.lock_2").exists())
     self.assertTrue((self.tmpdir/"lock1.lock_5").exists())
     self.assertTrue((self.tmpdir/"lock1.lock_10").exists())
     self.assertTrue((self.tmpdir/"lock1.lock_30").exists())
-    subprocess.run(["clean_up_old_job_locks", self.tmpdir, "--hours-old", str(1/3600), "--silent"], check=True)
     clean_up_old_job_locks(self.tmpdir, howold=datetime.timedelta(seconds=1), silent=True)
+    subprocess.run(["clean_up_old_job_locks", self.tmpdir, "--hours-old", str(1/3600), "--silent"], check=True)
     self.assertFalse((self.tmpdir/"lock1.lock_2").exists())
     self.assertFalse((self.tmpdir/"lock1.lock_5").exists())
     self.assertFalse((self.tmpdir/"lock1.lock_10").exists())
@@ -519,3 +518,139 @@ class TestJobLock(unittest.TestCase, contextlib.ExitStack):
     with JobLock(self.tmpdir/"lock6.lock") as lock6:
       self.assertFalse(lock6)
 
+  def testinvalidsqueue(self):
+    dummysqueue = """
+      #!/bin/bash
+      echo '
+           1234567
+      '
+    """.lstrip()
+    with open(self.tmpdir/"squeue", "w") as f:
+      f.write(dummysqueue)
+    (self.tmpdir/"squeue").chmod(0o0777)
+    os.environ["PATH"] = f"{self.tmpdir}:"+os.environ["PATH"]
+
+    with open(self.tmpdir/"lock1.lock", "w") as f:
+      f.write("SLURM 0 1234567")
+
+    with JobLock(self.tmpdir/"lock1.lock") as lock1:
+      self.assertFalse(lock1)
+
+  def testsqueueerror(self):
+    dummysqueue = """
+      #!/bin/bash
+      if [ $2 -eq 1234567 ]; then
+        echo "slurm_load_jobs error: Invalid job id specified"
+      elif [ $2 -eq 1234568 ]; then
+        echo "slurm_load_jobs error: Unable to contact slurm controller (connect failure)"
+      elif [ $2 -eq 1234569 ]; then
+        echo "slurm_load_jobs error: Socket timed out on send/recv operation"
+      else
+        : nothing
+      fi
+      exit 1
+    """.lstrip()
+
+    badcondorq = """
+      #!/bin/bash
+      echo "Can't find address for schedd"
+      exit 1
+    """.lstrip()
+    goodcondorq = """
+      #!/bin/bash
+      echo "
+        -- Schedd: my schedd
+         ID      OWNER            SUBMITTED     RUN_TIME ST PRI SIZE CMD
+         1234567.1
+         1234568.1
+      "
+    """.lstrip()
+    reallybadcondorq = """
+      #!/bin/bash
+      exit 1
+    """.lstrip()
+
+    with open(self.tmpdir/"squeue", "w") as f:
+      f.write(dummysqueue)
+    (self.tmpdir/"squeue").chmod(0o0777)
+    with open(self.tmpdir/"condor_q", "w") as f: f.write(goodcondorq)
+    (self.tmpdir/"condor_q").chmod(0o0777)
+    os.environ["PATH"] = f"{self.tmpdir}:"+os.environ["PATH"]
+
+    with open(self.tmpdir/"lock1.lock", "w") as f:
+      f.write("SLURM 0 1234567")
+    with open(self.tmpdir/"lock2.lock", "w") as f:
+      f.write("SLURM 0 1234568")
+    with open(self.tmpdir/"lock3.lock", "w") as f:
+      f.write("SLURM 0 1234567")
+    with open(self.tmpdir/"lock4.lock", "w") as f:
+      f.write("CONDOR 1234567 0")
+    with open(self.tmpdir/"lock5.lock", "w") as f:
+      f.write("CONDOR 1234567 0")
+    with open(self.tmpdir/"lock6.lock", "w") as f:
+      f.write("CONDOR 1234568 0")
+    with open(self.tmpdir/"lock7.lock", "w") as f:
+      f.write("CONDOR 1234567 0")
+    with open(self.tmpdir/"lock8.lock", "w") as f:
+      f.write("SLURM 0 1234567")
+    with open(self.tmpdir/"lock9.lock", "w") as f:
+      f.write("SLURM 0 1234569")
+    with open(self.tmpdir/"lock10.lock", "w") as f:
+      f.write("SLURM 0 1234567")
+    with open(self.tmpdir/"lock11.lock", "w") as f:
+      f.write("SLURM 0 1234570")
+    with open(self.tmpdir/"lock12.lock", "w") as f:
+      f.write("SLURM 0 1234567")
+
+    with JobLock(self.tmpdir/"lock1.lock") as lock:
+      self.assertTrue(lock)
+    with JobLock(self.tmpdir/"lock2.lock") as lock:
+      self.assertFalse(lock)
+    with JobLock(self.tmpdir/"lock3.lock") as lock:
+      #won't try to run squeue anymore because lock2 gave an unknown error
+      self.assertFalse(lock)
+
+    with open(self.tmpdir/"condor_q", "w") as f: f.write(reallybadcondorq)
+    with self.assertRaises(subprocess.CalledProcessError):
+      with JobLock(self.tmpdir/"lock4.lock") as lock:
+        pass
+    with open(self.tmpdir/"condor_q", "w") as f: f.write(goodcondorq)
+    with JobLock(self.tmpdir/"lock5.lock") as lock:
+      self.assertTrue(lock)
+    with open(self.tmpdir/"condor_q", "w") as f: f.write(badcondorq)
+    with JobLock(self.tmpdir/"lock6.lock") as lock:
+      self.assertFalse(lock)
+    with open(self.tmpdir/"condor_q", "w") as f: f.write(goodcondorq)
+    with JobLock(self.tmpdir/"lock7.lock") as lock:
+      self.assertFalse(lock)
+
+    clear_running_jobs_cache()
+
+    with JobLock(self.tmpdir/"lock8.lock") as lock:
+      #will now try again because the cache is cleared
+      self.assertTrue(lock)
+    with JobLock(self.tmpdir/"lock9.lock") as lock:
+      self.assertFalse(lock)
+    with JobLock(self.tmpdir/"lock10.lock") as lock:
+      self.assertFalse(lock)
+    clear_running_jobs_cache()
+    with self.assertRaises(subprocess.CalledProcessError):
+      with JobLock(self.tmpdir/"lock11.lock") as lock:
+        pass
+    with JobLock(self.tmpdir/"lock12.lock") as lock:
+      self.assertTrue(lock)
+
+def main():
+  parser = argparse.ArgumentParser()
+  parser.add_argument("--debug-log", action="store_true")
+  parser.add_argument("unittest_args", nargs="*")
+
+  args = parser.parse_args()
+  if args.debug_log:
+    TestJobLock.loglevel = logging.DEBUG
+
+  sys.argv[1:] = args.unittest_args
+  unittest.main()
+
+if __name__ == "__main__":
+  main()
